package de.uks.pmws1112.hexentanz.model;

import java.util.*;

public class Field
{
   public static final String PROPERTY_COLOR = "color";

   private String color;

   public void setColor (String value)
   {
      if (this.color == null || !this.color.equals(value))
      {
         String oldValue = this.color;
         this.color = value;
      }
   }

   public Field withColor (String value)
   {
      setColor (value);
      return this;
   }

   public String getColor ()
   {
      return this.color;
   }

   /**
    * <pre>
    *           0..1     finalGate     0..1
    * Field ------------------------- FinalField
    *           finalGateRev               finalGate
    * </pre>
    */
   public static final String PROPERTY_FINAL_GATE = "finalGate";

   private FinalField finalGate;

   public boolean setFinalGate (FinalField value)
   {
      boolean changed = false;

      if (this.finalGate != value)
      {
      
         FinalField oldValue = this.finalGate;
         Field source = this;
         if (this.finalGate != null)
         {
            this.finalGate = null;
            oldValue.setFinalGateRev (null);
         }
         this.finalGate = value;

         if (value != null)
         {
            value.setFinalGateRev (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Field withFinalGate (FinalField value)
   {
      setFinalGate (value);
      return this;
   }

   public FinalField getFinalGate ()
   {
      return this.finalGate;
   }

   /**
    * <pre>
    *           0..*     fields     0..1
    * Field ------------------------- Game
    *           fields               game
    * </pre>
    */
   public static final String PROPERTY_GAME = "game";

   private HexentanzGame game;

   public boolean setGame (HexentanzGame value)
   {
      boolean changed = false;

      if (this.game != value)
      {
      
         HexentanzGame oldValue = this.game;
         Field source = this;
         if (this.game != null)
         {
            this.game = null;
            oldValue.removeFromFields (this);
         }
         this.game = value;

         if (value != null)
         {
            value.addToFields (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Field withGame (HexentanzGame value)
   {
      setGame (value);
      return this;
   }

   public HexentanzGame getGame ()
   {
      return this.game;
   }

   /**
    * <pre>
    *           0..1     field     0..1
    * Field ------------------------- Field
    *           previous               next
    * </pre>
    */
   public static final String PROPERTY_NEXT = "next";
   
   private Field next;

   public boolean setNext (Field value)
   {
      boolean changed = false;

      if (this.next != value)
      {
      
         Field oldValue = this.next;
         Field source = this;
         if (this.next != null)
         {
            this.next = null;
            oldValue.setPrevious (null);
         }
         this.next = value;

         if (value != null)
         {
            value.setPrevious (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Field withNext (Field value)
   {
      setNext (value);
      return this;
   }

   public Field getNext ()
   {
      return this.next;
   }

   public static final String PROPERTY_NR = "nr";

   private int nr;

   public void setNr (int value)
   {
      if ( this.nr != value )
      {
         int oldValue = this.nr;
         this.nr = value;
      }
   }

   public Field withNr (int value)
   {
      setNr (value);
      return this;
   }

   public int getNr ()
   {
      return this.nr;
   }

   /**
    * <pre>
    *           0..1     field     0..1
    * Field ------------------------- Field
    *           next               previous
    * </pre>
    */
   public static final String PROPERTY_PREVIOUS = "previous";
   
   private Field previous;

   public boolean setPrevious (Field value)
   {
      boolean changed = false;

      if (this.previous != value)
      {
      
         Field oldValue = this.previous;
         Field source = this;
         if (this.previous != null)
         {
            this.previous = null;
            oldValue.setNext (null);
         }
         this.previous = value;

         if (value != null)
         {
            value.setNext (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Field withPrevious (Field value)
   {
      setPrevious (value);
      return this;
   }

   public Field getPrevious ()
   {
      return this.previous;
   }

   /**
    * <pre>
    *           0..1     field     0..*
    * Field ------------------------- Witch
    *           field               witches
    * </pre>
    */
   public static final String PROPERTY_WITCHES = "witches";

   private HashSet<Witch> witches;

   public Set<? extends Witch> getWitches()
   {
      return ((this.witches == null)
              ? Collections.EMPTY_SET
              : Collections.unmodifiableSet(this.witches));
   }

   public boolean addToWitches (Witch value)
   {
      boolean changed = false;

      if (value != null)
      {
         if (this.witches == null)
         {
            this.witches = new HashSet<Witch> ();
         }
      
         changed = this.witches.add (value);
         if (changed)
         {
            value.setField (this);
         }
      
      }
      return changed;
   }

   public Field withWitches (Witch value)
   {
      addToWitches (value);
      return this;
   }

   public Field withoutWitches (Witch value)
   {
      removeFromWitches (value);
      return this;
   }


   public boolean removeFromWitches (Witch value)
   {
      boolean changed = false;

      if ((this.witches != null) && (value != null))
      {
      
         changed = this.witches.remove (value);
         if (changed)
         {
            value.setField (null);
         }
      
      }
      return changed;
   }

   public void removeAllFromWitches (){
   
      Witch tmpValue;
      Iterator<? extends Witch> iter = this.iteratorOfWitches ();
      while (iter.hasNext ())
      {
         tmpValue = (Witch) iter.next ();
         this.removeFromWitches (tmpValue);
      }
   
   }

   public boolean hasInWitches (Witch value)
   {
      return ((this.witches != null) &&
              (value != null) &&
              this.witches.contains (value));
   }

   public Iterator<? extends Witch> iteratorOfWitches ()
   {
      return ((this.witches == null)
              ? new Iterator<Witch>()
            	{
					@Override
					public boolean hasNext() 
					{
						return false;
					}
	
					@Override
					public Witch next()
					{
						return null;
					}
	
					@Override
					public void remove() {}
				}
              : this.witches.iterator ());
   }
   
   public int sizeOfWitches ()
   {
      return ((this.witches == null)
              ? 0
              : this.witches.size ());
   }

   public void removeYou()
   {
      this.setFinalGate (null);
      this.setGame (null);
      this.setNext (null);
      this.setPrevious (null);
      this.removeAllFromWitches ();
   }
}


